/*
Copyright 2009,2010 by Fabian Schwartau
E-Mail: fabian@opencode.eu

This file is part of HM2PC.

HM2PC is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HM2PC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with HM2PC  If not, see <http://www.gnu.org/licenses/>.
*/

#include "uart.h"

void uart_init()
{
  uint8_t sreg = SREG;
  // init baud rate
  UBRRH = (uint8_t)( UART_UBRR_CALC( UART_BAUD_RATE, F_CPU ) >> 8 );
  UBRRL = (uint8_t)( UART_UBRR_CALC( UART_BAUD_RATE, F_CPU )      );
  
  cli();  // turn off interrupts for a moment
  
  UCSRB = (1<<TXEN) | (1<<RXEN);			// UART TX einschalten
  UCSRC = (1<<URSEL) | (1<<UCSZ0) | (1<<UCSZ1);	// Asynchron 8N1
  
  // read trash data from uart...
  do
  {
    UDR;
  }
  while (UCSRA & (1 << RXC));
  
  UCSRA = (1 << RXC) | (1 << TXC); // | (1<<U2X);  // U2X = double speed
  
  SREG = sreg;
  sei(); // enable interrupts again
  
  return;
}

void uart_putc(uint8_t buchstabe)
{
  while (!(UCSRA & (1<<UDRE)));  /* warten bis Senden moeglich */
  UDR = buchstabe;
}

void uart_putcc(const char* String)
{
  while(*String)
  {
    uart_putc(*String);
    String++;
  }
}

void uart_puti(uint32_t zahl)
{
  char s[7];
  itoa(zahl, s, 10);
  uart_putcc(s);
}

uint8_t uart_getc()
{
	while(!(UCSRA & (1<<RXC))); // wait for received byte
	return UDR;
}

int uart_get_noblock()
{
	if((UCSRA & (1<<RXC)))
		return (int)UDR;
	else
		return -1;
}
