/*
Copyright 2009,2010 by Fabian Schwartau
E-Mail: fabian@opencode.eu

This file is part of HM2PC.

HM2PC is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HM2PC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with HM2PC.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MainWnd.h"
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>

MainWnd::MainWnd( QWidget* parent /*= 0*/, Qt::WFlags flags /*= 0*/ ) : 
QMainWindow(parent, flags),
renderArea(this)
{
	setupUi(this);
	timerRefresh = new QTimer(this);
	connect(timerRefresh, SIGNAL(timeout()), this, SLOT(onTimerRefresh()));

	statusBar = new QStatusBar(this);
	this->setStatusBar(statusBar);

	screenData=new char[2048];
	for(int i=0; i<2048; i++)
		screenData[i]=0;
	renderArea.setScreenData(screenData);
	renderArea.show();
}


MainWnd::~MainWnd()
{
	delete[] screenData;
}

void MainWnd::onButtonOpen()
{
	int error;
	if((error=hameg.openCom(editCOMPort->text(), editBaud->text()))!=OK)
	{
		QMessageBox::information(this, "ERROR", "Failed to open COM port: "+QString::number(error));
		return;
	}
	if(!hameg.checkConnection())
		QMessageBox::information(this, "Interface not responding", "This interface is not responding!");
	time.start();
	//received=0;
	btOpen->setEnabled(false);
	btClose->setEnabled(true);
	btRefresh->setEnabled(true);
	radioRefreshAuto->setEnabled(true);
	radioRefreshManual->setEnabled(true);
	btSaveTable->setEnabled(true);
	btSaveImage->setEnabled(true);
	if(radioRefreshAuto->isChecked())
		timerRefresh->start();
}

void MainWnd::onButtonClose()
{
	btOpen->setEnabled(true);
	btClose->setEnabled(false);
	btRefresh->setEnabled(false);
	radioRefreshAuto->setEnabled(false);
	radioRefreshManual->setEnabled(false);
	btSaveTable->setEnabled(false);
	btSaveImage->setEnabled(false);
	hameg.closeCom();
	timerRefresh->stop();
}

void MainWnd::onTimerRefresh()
{
	refreshScreen();
}

void MainWnd::resizeEvent(QResizeEvent* event)
{
	int width=event->size().width()-40;
	int height=event->size().height()-190;
	if(width<=0)
		width=1;
	if(height<=0)
		height=1;
	renderArea.setGeometry(20, 150, width, height);
}

void MainWnd::onButtonSaveImage()
{
	QString fileName=QFileDialog::getSaveFileName(this, "Save dialog", "", "(*.png);;(*.jpg);;(*.bmp);;(*.tif)");
	if(fileName=="")
		return;
	int width, height;
	if(radioImageFull->isChecked())
	{
		width=2048;
		height=256;
	}
	else if(radioImagePort->isChecked())
	{
		width=renderArea.width();
		height=renderArea.height();
	}
	else if(radioImageCustom->isChecked())
	{
		bool isOK;
		width=lineImageWidth->text().toUInt(&isOK);
		if(!isOK)
		{
			width=renderArea.width();
			statusBar->showMessage("This is not a number, correct the size settings! Used view port!", 3000);
		}
		height=lineImageHeight->text().toUInt(&isOK);
		if(!isOK)
		{
			height=renderArea.height();
			statusBar->showMessage("This is not a number, correct the size settings! Used view port!", 3000);
		}
	}
		renderArea.saveToImage(fileName, width, height);
}

void MainWnd::onImageSaveTypeChanged()
{
	if(radioImageFull->isChecked())
	{
		lineImageHeight->setEnabled(false);
		lineImageWidth->setEnabled(false);
	}
	else if(radioImagePort->isChecked())
	{
		lineImageHeight->setEnabled(false);
		lineImageWidth->setEnabled(false);
	}
	else  // Custom
	{
		lineImageHeight->setEnabled(true);
		lineImageWidth->setEnabled(true);
	}
}

void MainWnd::onRefreshModeChanged()
{
	unsigned int refreshTime;
	bool isOK;
	refreshTime=editRefreshRate->text().toUInt(&isOK);
	if(!isOK)
		QMessageBox::information(this, "Error switching mode", "Failed to switch the mode, the refresh rate does not seem to be a number!");
	if(refreshTime>1000 || refreshTime<=1)
		QMessageBox::information(this, "Error switching mode", "Failed to switch the mode, the refresh rate is to high!\nChoose something less or eaqual 1000");
	if(!isOK || refreshTime>1000 || refreshTime<1)
	{
		if(radioRefreshAuto->isChecked())
		{
			radioRefreshManual->setChecked(true);
			radioRefreshAuto->setChecked(false);
		}
		else
		{
			radioRefreshManual->setChecked(false);
			radioRefreshAuto->setChecked(true);
		}
		return;
	}

	if(radioRefreshAuto->isChecked())
	{
		editRefreshRate->setEnabled(true);
		btRefresh->setEnabled(false);
		timerRefresh->setInterval(1000/refreshTime);
		timerRefresh->start();
	}
	else
	{
		editRefreshRate->setEnabled(false);
		btRefresh->setEnabled(true);
		timerRefresh->stop();
	}
}

void MainWnd::onRefreshRateChanged(QString newValue)
{
	unsigned int refreshTime;
	bool isOK;
	refreshTime=editRefreshRate->text().toUInt(&isOK);
	if(!isOK)
		statusBar->showMessage("The refresh rate does not seem to be a number!");
	else if(refreshTime>1000 || refreshTime<10)
		statusBar->showMessage("The refresh rate is to high!\nChoose something less or equal 1000");
	else
	{
		statusBar->showMessage("Refresh rate changed", 2000);
		timerRefresh->setInterval(1000/refreshTime);
	}
}

void MainWnd::onButtonRefresh()
{
	refreshScreen();
}

void MainWnd::refreshScreen()
{

	if(!hameg.getSingleShot(screenData))
	{
		QMessageBox::information(this, "Error getting the screenshot", "Failed to get the screenshot");
		return;
	}
	renderArea.setScreenData(screenData);
}